export declare type ActionCallbacks = Callbacks<ActionReturnValue>;

export declare type ActionReturnValue = ExecReturnValue | CompleteReturnValue | InstallReturnValue | FormatReturnValue | JSFnCallReturnValue;

export declare enum ActionType {
    EXEC = 0,
    COMPLETE = 1,
    INSTALL = 2,
    FORMAT = 3,
    JS_FN_CALL = 4
}

export declare type Callback<T> = {
    resolve: (value: T) => void;
    reject: (value: string) => void;
};

declare type Callbacks<T> = Record<CommandUniqueId, Callback<T>>;

export declare enum ChannelSetupStatus {
    READY = 0
}

export declare type ChannelTransmitData = {
    id: CommandUniqueId;
    action: ActionType;
    data: Payload | Response_2;
};

export declare type CommandUniqueId = number;

export declare const complete: {
    repl(payload: CompletePayload): Promise<CompletionResults>;
};

export declare type CompletePayload = {
    code: string;
    line?: number;
    column?: number;
};

export declare type CompleteResponse = {
    result: CompletionResults;
    id: CommandUniqueId;
    error: string | null;
};

export declare type CompleteReturnValue = Omit<CompleteResponse, 'id'>;

declare type CompletionMatch = CompletionResult;

export declare type CompletionResult = {
    name: string;
    type: string;
    description: string;
    full_name: string;
};

export declare type CompletionResults = {
    matches: CompletionMatch[];
};

export declare type ComplexPayload = {
    type: PayloadType;
    name: string;
};

export declare type Context = Record<string, unknown | ComplexPayload>;

export declare function exec(payload: ExecPayload): Promise<ExecReturnValue>;

export declare type ExecPayload = {
    code: string;
    context?: Context;
};

export declare type ExecResponse = {
    id: CommandUniqueId;
    error: string | null;
    result: string | null;
    stdout: string | null;
    stderr: string | null;
};

export declare type ExecReturnValue = Omit<ExecResponse, 'id'>;

export declare function format(payload: FormatPayload): Promise<FormatReturnValue>;

export declare type FormatPayload = {
    code: string;
};

export declare type FormatResponse = {
    id: CommandUniqueId;
    result: string | null;
    error: string | null;
};

export declare type FormatReturnValue = Omit<FormatResponse, 'id'>;

export declare const init: () => Promise<Worker>;

export declare function install(packages: string[]): Promise<InstallReturnValue>;

export declare type InstallPayload = {
    packages: string[];
};

export declare type InstallResponse = {
    id: CommandUniqueId;
    success: boolean;
    error: string | null;
};

export declare type InstallReturnValue = Omit<InstallResponse, 'id'>;

export declare function interrupt(): void;

export declare type JSCallbacks = Callbacks<unknown>;

export declare type JSFnCallPayload = {
    args: unknown[];
    name: string;
};

export declare type JSFnCallResponse = {
    id: CommandUniqueId;
    result: unknown;
    error?: string | null;
};

export declare type JSFnCallReturnValue = Omit<JSFnCallResponse, 'id'>;

export declare type JSFunctions = {
    [name: string]: Function;
};

export declare type MainModuleState = {
    config: object;
    pyodideWorker: Worker | null;
    callbacks: ActionCallbacks;
    commandUniqueId: CommandUniqueId;
    jsFunctions: JSFunctions;
    interruptBuffer: Uint8Array | null;
};

export declare type Payload = ExecPayload | CompletePayload | InstallPayload | FormatPayload | JSFnCallPayload;

export declare enum PayloadType {
    FN = "__function__"
}

declare type Response_2 = ExecResponse | CompleteResponse | InstallResponse | FormatResponse;
export { Response_2 as Response }

export declare type WorkerModuleState = {
    callbacks: JSCallbacks;
    commandUniqueId: CommandUniqueId;
};

export { }
